/*
 * Copyright (C) 2012 by
 *   MetraLabs GmbH (MLAB), GERMANY
 * and
 *   Neuroinformatics and Cognitive Robotics Labs (NICR) at TU Ilmenau, GERMANY
 * All rights reserved.
 *
 * Contact: info@mira-project.org
 *
 * Commercial Usage:
 *   Licensees holding valid commercial licenses may use this file in
 *   accordance with the commercial license agreement provided with the
 *   software or, alternatively, in accordance with the terms contained in
 *   a written agreement between you and MLAB or NICR.
 *
 * GNU General Public License Usage:
 *   Alternatively, this file may be used under the terms of the GNU
 *   General Public License version 3.0 as published by the Free Software
 *   Foundation and appearing in the file LICENSE.GPL3 included in the
 *   packaging of this file. Please review the following information to
 *   ensure the GNU General Public License version 3.0 requirements will be
 *   met: http://www.gnu.org/copyleft/gpl.html.
 *   Alternatively you may (at your option) use any later version of the GNU
 *   General Public License if such license has been publicly approved by
 *   MLAB and NICR (or its successors, if any).
 *
 * IN NO EVENT SHALL "MLAB" OR "NICR" BE LIABLE TO ANY PARTY FOR DIRECT,
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
 * THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF "MLAB" OR
 * "NICR" HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * "MLAB" AND "NICR" SPECIFICALLY DISCLAIM ANY WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND "MLAB" AND "NICR" HAVE NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS OR MODIFICATIONS.
 */

/**
 * @file AxesVisualization.C
 *    Implementation of AxesVisualization.
 *
 * @author Tim Langner
 * @date   2010/12/30
 */

#include <visualization/3d/AxesVisualization.h>

#include <widgets/OgreUtils.h>
#include <OGRE/OgreSceneManager.h>

namespace mira {

///////////////////////////////////////////////////////////////////////////////

void AxesVisualization::setAxisLength(float length)
{
	mAxisLength = length;
	if ( mAxes != NULL )
		mAxes->setLength(length);
}

void AxesVisualization::setAxisRadius(float radius)
{
	mAxisRadius = radius;
	if ( mAxes != NULL )
		mAxes->setRadius(radius);
}

void AxesVisualization::setShowArrows(bool show)
{
	mShowArrows = show;
	if ( mAxes != NULL )
		mAxes->showArrows(show);
}

void AxesVisualization::setupScene(IVisualization3DSite* site)
{
	Ogre::SceneManager* sceneManager = site->getSceneManager();
	mAxes = new AxesObject(site->getSceneManager(), sceneManager->getRootSceneNode());
	mAxes->setLength(mAxisLength);
	mAxes->setRadius(mAxisRadius);
	mAxes->showArrows(mShowArrows);
	mAxes->setVisible(isEnabled());
}

void AxesVisualization::setEnabled(bool enabled)
{
	Visualization3D::setEnabled(enabled);
	if (mAxes != NULL)
		mAxes->setVisible(enabled);
}

///////////////////////////////////////////////////////////////////////////////

}

MIRA_CLASS_SERIALIZATION( mira::AxesVisualization, mira::Visualization3D);
