/*
 * Copyright (C) 2012 by
 *   MetraLabs GmbH (MLAB), GERMANY
 * and
 *   Neuroinformatics and Cognitive Robotics Labs (NICR) at TU Ilmenau, GERMANY
 * All rights reserved.
 *
 * Contact: info@mira-project.org
 *
 * Commercial Usage:
 *   Licensees holding valid commercial licenses may use this file in
 *   accordance with the commercial license agreement provided with the
 *   software or, alternatively, in accordance with the terms contained in
 *   a written agreement between you and MLAB or NICR.
 *
 * GNU General Public License Usage:
 *   Alternatively, this file may be used under the terms of the GNU
 *   General Public License version 3.0 as published by the Free Software
 *   Foundation and appearing in the file LICENSE.GPL3 included in the
 *   packaging of this file. Please review the following information to
 *   ensure the GNU General Public License version 3.0 requirements will be
 *   met: http://www.gnu.org/copyleft/gpl.html.
 *   Alternatively you may (at your option) use any later version of the GNU
 *   General Public License if such license has been publicly approved by
 *   MLAB and NICR (or its successors, if any).
 *
 * IN NO EVENT SHALL "MLAB" OR "NICR" BE LIABLE TO ANY PARTY FOR DIRECT,
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
 * THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF "MLAB" OR
 * "NICR" HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * "MLAB" AND "NICR" SPECIFICALLY DISCLAIM ANY WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND "MLAB" AND "NICR" HAVE NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS OR MODIFICATIONS.
 */

/**
 * @file Url.C
 *    Implementation of Url.h.
 *
 * @author Ronny Stricker
 * @date   2012/12/03
 */
#include "core/Url.h"

using namespace std;

namespace mira {

///////////////////////////////////////////////////////////////////////////////

Url operator / ( Url const& lhs, Url const& rhs )
{
	if ( lhs.size() == 0)
		return rhs;
	Url tReturn = lhs;
	while ( tReturn.length() > 0 && tReturn[tReturn.length()-1] == '/' ) {
		tReturn = tReturn.substr(0,tReturn.length()-1);
	}
	if ( rhs.length()>0 ) {
		Url tRhs = rhs;
		while ( tRhs.length() > 0 && tRhs[ 0 ] == '/' ) {
			tRhs = tRhs.substr(1);
		}
		if ( tReturn.size() > 0 )
			tReturn += '/';
		if ( tRhs.length() > 0 )
			tReturn.append( tRhs );
	}
	return tReturn;
}

Url getParentUrl( Url const& url )
{
	using namespace std;
	if ( url.find('/') != std::string::npos ) {
		return url.substr(0,url.rfind('/'));
	}
	return "";
}

Url getFilename( Url const& url )
{
	using namespace std;
	if ( url.find('/') != std::string::npos ) {
		return url.substr(url.rfind('/')+1);
	}
	return url;
}

///////////////////////////////////////////////////////////////////////////////

}
